// MyApp.cc

#include <iostream.h>
#include <complex.h>
#include "ocs.hh"

#include "MyStartup.hh"

// Declare all functions in this file (except for the function 
// MyApplication) here:

void BackToBackNoiseFreeSystem(StartupExample * MyStartup);

extern ofstream LogFile;

// #######################################################

// ## Application Wrapper

void MyApplication(int argc, char *argv[]) 
{

  // Construct an instance  MyStartup of the class StartupExample
  // See MyStartup.hh for the definition of this class 

  // Startup.in is the name of the input parameter file for the class
  // StartupExample

 StartupExample MyStartup("Startup.in",argc,argv);

 // Read O'Reilly pp. 110-115 for info on switch statements. 
 // switch must take an integer argument, in this case
 // MyStartup.GetSimulationType()

  switch(MyStartup.GetSimulationType())
   {
    
    case 1:
      BackToBackNoiseFreeSystem(&MyStartup);
      break;

    default:
      cerr << "Invalid choice of application in SimType" << endl;
      exit(1);
          
   } // end switch

} // ## end MyApplication

// ########################################################

void BackToBackNoiseFreeSystem(StartupExample * MyStartup)
{

  // Read about strings in O'Reilly page 49

  string Job = MyStartup->GetJobPrefix();

  LogFileSeparator();
  LogFile << "BackToBackSystem" << endl;

  // Construct instances RNG and Signal of the classes 
  // RanNumGen and OptSignal, repectively 

  RanNumGen RNG("RanNumGen.in");
  OptSignal Signal("OptSignal.in", &RNG);
  
  // Write the initial optical signal power in time and in frequency to files 

  int OmitDCFlag = 0;

  Signal.WriteFileTimePower(Job + "tInOpt.dat");
  Signal.WriteFileFreqPower(Job + "fInOpt.dat",OmitDCFlag);

  // Construct the components of a receiver subsystem

  OptFilter oOptFilter("OptFilter.in",&Signal);
  Photodetector  oPhotodetector("Photodetector.in",&Signal); 
  ElecFilter oElecFilter("ElecFilter.in",&oPhotodetector);
  ElecSignalStat oElecSignalStat(&Signal,&oPhotodetector);   

  // Pass the optical signal through the receiver 

   oOptFilter.FilterOptSignal();
   oPhotodetector.DetectOptSignal();   
   oElecFilter.FilterElecSignal();
   oElecSignalStat.GetSingleStringElecSignalStat();

   cout << "Back to Back Eye Opening [mV] = " 
	<< 1e3*oElecSignalStat.GetMinAmplitudeMargin() << endl;

   // Write the electrical signal power in time and in frequency  to a file

   oPhotodetector.WriteFileTimePower(Job + "tReceivedCurrent.dat");   
   oPhotodetector.WriteFileFreqPower(Job + "fReceivedCurrent.dat",OmitDCFlag);

   // Write out the Eye diagram

   oElecSignalStat.WriteFileEyeDiagram(Job + "Eye.dat"); 

   LogFile << "THE END" << endl;
   cout << "THE END" << endl;

} // ## end Back2BackNoiseFreeSystem

// ############################################
