// Statistics.cc

/*

 Statistics.cc contains the source code for the Statistics class
 which consists of the definitions of all of the class functions
 that are not already defined in Statistics.hh

*/

#include<stdlib.h>
#include<math.h>



// Always include the header file for the class!!

#include "Statistics.hh"

Statistics::Statistics(int arrayLengthIn, double *arrayIn)
  {

    // Initialize all private data members of the class

    arrayLength = arrayLengthIn;
    array = arrayIn;

    // To call a class function from within the class you just 
    // call it like you call a function in C

    ComputeMinimum(); 
    ComputeMaximum();
    ComputeMean();
    ComputeVariance();

    standardDeviation = sqrt(variance);

    // Dynamically allocate memory of size arrayLength for sortedArray

    sortedArray = new double[arrayLength];

    ComputeMedian();

  }// ## end Statistics constructor


// ########################################################

Statistics::~Statistics()
  {

    // Delete memory that was dynamically allocated in the constructor

    delete [] sortedArray;    
  }
  
// ########################################################

void Statistics::ComputeMinimum(void)
{

  minimum = array[0];

  for(int i=0;i<arrayLength;i++)
    {
      if(array[i] < minimum)
         minimum = array[i];
    }

} // ## end ComputeMinimum

// ########################################################

void Statistics::ComputeMaximum(void)
{

  maximum = array[0];

  for(int i=0;i<arrayLength;i++)
    {
      if(array[i] > maximum)
         maximum = array[i];
    }

} // ## end ComputeMaximum

// ########################################################

void Statistics::ComputeMean(void)
{

  mean = 0.0;

  for(int i=0;i<arrayLength;i++)
     mean += array[i];

  mean = mean/(double(arrayLength));

} // ## end ComputeMean

// #####################################################

void Statistics::ComputeMeanOfSquares(void)
{

  meanOfSquares = 0.0;

  for(int i=0;i<arrayLength;i++)
    meanOfSquares += array[i]*array[i];

  meanOfSquares = meanOfSquares/(double(arrayLength));

} // ## end ComputeMeanSquares

// ########################################################

void Statistics::ComputeVariance(void)
{  
 // This gives an "unbiased" computation of variance

  // Make sure we compute mean and meanOfSquares first!

  ComputeMean();
  ComputeMeanOfSquares();

  variance =  double(arrayLength)*(meanOfSquares-mean*mean)/ 
              double(arrayLength-1.0);

} // ## end ComputeVariance

// ########################################################

void Statistics::ComputeMedian(void)
{
  
    SelectionSort();

    if(arrayLength%2) // true if arrayLength is odd
      median = sortedArray[(arrayLength+1)/2 - 1];
    else // arrayLength even
      median = (sortedArray[arrayLength/2 - 1] + 
		sortedArray[arrayLength/2])/2.0;

} // ## end Median

// #############################################

void Statistics::SelectionSort(void)
{

  for(int ii=0;ii<arrayLength;ii++)
    sortedArray[ii] = array[ii];

  for(int SubLength = arrayLength; SubLength >=1; SubLength --)
    {
      int Index = MaximumIndex(SubLength);

      if(Index < SubLength - 1) // If Index == SubLength do nothing
	{
          double Temp = sortedArray[SubLength-1];
          sortedArray[SubLength-1] = sortedArray[Index];
          sortedArray[Index] = Temp;
	} // ## end if

    } // ## end for

} // ## end SelectionSort

// ##############################################

int Statistics::MaximumIndex(int SubLength)
{

  double Max = array[0];
  int MaxIndex = 0;

  for(int ii=0;ii<SubLength;ii++)
    {
      if(array[ii] > Max)
	{
          Max = array[ii];
          MaxIndex = ii;
	}
    }

  return MaxIndex;

} // ## end MaximumIndex

// ##########################################







